#ifndef CGAMESTATE_H
#define CGAMESTATE_H

// Irrlicht includes
#include <irrlicht/irrlicht.h>

using namespace irr;

using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward declaration of game state manager
class CStateManager;

// All of the game states available
enum GAME_STATE { GS_LOADING,
                  GS_PLAYING };

// Abstract base class for all other game states
// Notice that this class derives from the event receiver
// class - that's because each game state needs to have
// its own event management code
class CGameState : public IEventReceiver
{
    public:

        // Constructor
        CGameState(CStateManager * pManager);
        virtual ~CGameState() {}

        //-------------------
        // Utility functions
        //-------------------

        // Indicates to the manager that we want to
        // switch the current game state to something else
        IrrlichtDevice * GetDevice();
        void SwitchState(GAME_STATE);

        void ClearScreen();
        void Done();

        //---------------------------------
        // Each of these MAY be overridden
        // in the derived class
        //---------------------------------
        virtual bool OnEvent(const SEvent& event) { return false; }

        //-----------------------------------
        // Each of these MUST be implemented
        // in the derived classes
        //-----------------------------------

        // Renders the current frame
        virtual void RenderFrame(u32)=0;

    protected:

        // The device
        CStateManager * m_manager;
};

#endif
