#ifndef CPLAYINGSTATE_H
#define CPLAYINGSTATE_H

#include "CGameState.h"

// OpenTracks includes
#include "CGround.h"
#include "CSkybox.h"
#include "CTrackManager.h"
#include "CTrainManager.h"

// Enumerations
enum INPUT_MODE { IM_SELECT=0,
                  IM_PLACE_TRACK,
                  IM_PLACE_TRAIN };

enum TRACK_CLASS { TC_STRAIGHT,
                   TC_CURVE };

class CPlayingState : public CGameState
{
    public:

        CPlayingState(CStateManager *);
        ~CPlayingState();

        bool OnEvent(const SEvent&);
        void RenderFrame(u32);

    private:

        // Utility functions
        bool KeyEvent(const SEvent&);
        bool MouseEvent(const SEvent&);
        bool GUIEvent(const SEvent&);

        // GUI creation / management functions
        void CreateToolbar();
        void CreateTrackWindow();
        void CreateTrainWindow();

        // Mesh functions
        void UpdateGhostItems();

        // Input manager
        void ProcessInput(u32);

        // We have a few things to keep track of here...

        // Screen objects and pointers
        ICameraSceneNode * m_camera;
        IGUIToolBar * m_toolbar;

        // State information
        INPUT_MODE m_mode;

        // Physical geometry
        CGround m_ground;
        CSkybox m_sky;

        // Track and train managers
        CTrackManager m_track_manager;
        CTrainManager m_train_manager;

        // Ghost objects
        IMeshSceneNode * m_straight_ghost_node;
        IMeshSceneNode * m_curve_ghost_node;
        IMeshSceneNode * m_engine_ghost_node;

        // GUI dialogs
        IGUIWindow * m_dialog_edittrack;
        IGUIWindow * m_dialog_addtrain;

        // Camera manipulation information
        u32 m_previous_time;
        float m_rotation;
        float m_radius;

        // Key information
        bool m_up, m_down, m_left, m_right;

        // Mouse information
        bool m_r_down;
        float m_x_diff, m_y_diff;
        s32 m_x_cur,  m_y_cur;
        s32 m_x_prev, m_y_prev;
        line3df m_cursor_ray;  // for ray-picking

        // Track placement information
        TRACK_CLASS m_current_class;
        TRACK_TYPE  m_current_track;
};

#endif
