#ifndef CSKIN_H
#define CSKIN_H

#include <irrlicht/irrlicht.h>

using namespace irr;

using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

class CSkin : public IGUISkin
{
    public:

        // Constructor and destructor
        CSkin(IrrlichtDevice *);
        ~CSkin();

        // We need to override a few functions here
		virtual SColor getColor(EGUI_DEFAULT_COLOR color) const;
		virtual void setColor(EGUI_DEFAULT_COLOR which, SColor newColor);
		virtual s32 getSize(EGUI_DEFAULT_SIZE size) const;
		virtual const wchar_t* getDefaultText(EGUI_DEFAULT_TEXT text) const ;
		virtual void setDefaultText(EGUI_DEFAULT_TEXT which, const wchar_t* newText);
		virtual void setSize(EGUI_DEFAULT_SIZE which, s32 size);
		virtual IGUIFont* getFont(EGUI_DEFAULT_FONT which=EGDF_DEFAULT) const;
		virtual void setFont(IGUIFont* font, EGUI_DEFAULT_FONT which=EGDF_DEFAULT);
		virtual IGUISpriteBank* getSpriteBank() const;
		virtual void setSpriteBank(IGUISpriteBank* bank);
		virtual u32 getIcon(EGUI_DEFAULT_ICON icon) const;
		virtual void setIcon(EGUI_DEFAULT_ICON icon, u32 index);
		virtual void draw3DButtonPaneStandard(IGUIElement* element,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0);
		virtual void draw3DButtonPanePressed(IGUIElement* element,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0);
		virtual void draw3DSunkenPane(IGUIElement* element,
			SColor bgcolor, bool flat, bool fillBackGround,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0);
		virtual core::rect<s32> draw3DWindowBackground(IGUIElement* element,
			bool drawTitleBar, SColor titleBarColor,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0,
			core::rect<s32>* checkClientArea=0);
		virtual void draw3DMenuPane(IGUIElement* element,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0);
		virtual void draw3DToolBar(IGUIElement* element,
			const core::rect<s32>& rect,
			const core::rect<s32>* clip=0);
		virtual void draw3DTabButton(IGUIElement* element, bool active,
			const core::rect<s32>& rect, const core::rect<s32>* clip=0, EGUI_ALIGNMENT alignment=EGUIA_UPPERLEFT);
		virtual void draw3DTabBody(IGUIElement* element, bool border, bool background,
			const core::rect<s32>& rect, const core::rect<s32>* clip=0, s32 tabHeight=-1, EGUI_ALIGNMENT alignment=EGUIA_UPPERLEFT );
		virtual void drawIcon(IGUIElement* element, EGUI_DEFAULT_ICON icon,
			const position2di position, u32 starttime=0, u32 currenttime=0,
			bool loop=false, const core::rect<s32>* clip=0);
		virtual void draw2DRectangle(IGUIElement* element, const SColor &color,
			const core::rect<s32>& pos, const core::rect<s32>* clip );

		virtual EGUI_SKIN_TYPE getType() const { return EGST_UNKNOWN; }

    private:

        // Internal utility functions
        void DrawRoundedBox(const core::rect<s32>& rect, const SColor& color) const;

        // Pointer to the video driver
        IrrlichtDevice * m_device;

        // The normal sized font
        IGUIFont * m_normal_font;

        // The miniature font
        IGUIFont * m_mini_font;

        // Our GUI texture and stuff
        ITexture * m_tex;
        IGUISpriteBank * m_sprites;
};

#endif
