#ifndef CTRACK_H
#define CTRACK_H

// Irrlicht includes
#include <irrlicht/irrlicht.h>

using namespace irr;

using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Standard includes
#include <string>

// Track types
enum TRACK_TYPE { TT_VERTICAL_STRAIGHT=0,
                  TT_HORIZONTAL_STRAIGHT,
                  TT_CURVE_NE,
                  TT_CURVE_SE,
                  TT_CURVE_SW,
                  TT_CURVE_NW,

                  TT_CURVE_NE_OTHER, // these represent the other
                  TT_CURVE_SE_OTHER, // end of the track piece
                  TT_CURVE_SW_OTHER,
                  TT_CURVE_NW_OTHER,

                  TT_NUM_PIECES };  // this last entry is used
                                    // to determine the number of pieces

// Directions
enum DIRECTION { D_NORTH=0,
                 D_EAST,
                 D_SOUTH,
                 D_WEST };

class CTrackOrientationInfo
{
    public:

        // Track length in units
        float length;

        // Translation
        float fTrans_x;
        float fTrans_y;
        float fTrans_z;

        // Rotation
        float fRot_x;
        float fRot_y;
        float fRot_z;

        int iOffsetX; // The next piece of track
        int iOffsetZ; // from the current offset
};

// This class represents an actual physical
// piece of track on the grid
class CTrack
{
    public:

        CTrack(IrrlichtDevice *,float,float,float,int,int);
        ~CTrack();

        void SetPiece(TRACK_TYPE,IMesh *);
        TRACK_TYPE GetPiece() { return m_type; }

        float GetXPos() { return m_x; }
        float GetZPos() { return m_z; }

        int GetXOffset() { return m_offset_x; }
        int GetZOffset() { return m_offset_z; }

    private:

        TRACK_TYPE m_type;

        IMeshSceneNode * m_node; // the scene node

        float m_x, m_y, m_z;        // the location of this piece
        int m_offset_x, m_offset_z; // The offset of this piece
};

// Track information
extern CTrackOrientationInfo g_trackOrientationInfo[];

#endif
