#include "CSkin.h"

CSkin::CSkin(IrrlichtDevice * pDevice)
{
    m_device = pDevice;
    m_device->grab();

    IGUIEnvironment * pGUI = m_device->getGUIEnvironment();

    m_normal_font = pGUI->getFont(DATA_DIR "/fonts/arial/arial.xml");
    m_mini_font   = pGUI->getFont(DATA_DIR "/fonts/arialmini/arialmini.xml");

    // Now load our texture
    m_tex = pDevice->getVideoDriver()->getTexture(DATA_DIR "/textures/gui.png");

    // Create and set up the sprite bank
    m_sprites = pGUI->addEmptySpriteBank("Bob");
    m_sprites->addTexture(m_tex);

    // Now create the rectangles
    core::array<core::rect<s32> > & rectangles = m_sprites->getPositions();
    rectangles.push_back(core::rect<s32>(16,0,32,16));        // 0 - close button
    rectangles.push_back(core::rect<s32>(32,0,48,16));        // 0 - drop-down button

    // Now create the sprites
    core::array<SGUISprite>& sprite_array = m_sprites->getSprites();

    for(int i=0;i<2;++i)
    {
        SGUISpriteFrame new_frame;
        new_frame.rectNumber = i;
        new_frame.textureNumber = 0;

        SGUISprite new_sprite;
        new_sprite.Frames.push_back(new_frame);
        new_sprite.frameTime = 0;

        sprite_array.push_back(new_sprite);
    }
}

CSkin::~CSkin()
{
    if(m_sprites) m_sprites->drop();
    if(m_normal_font) m_normal_font->drop();
    if(m_mini_font) m_mini_font->drop();
    if(m_tex) m_tex->drop();
}

SColor CSkin::getColor(EGUI_DEFAULT_COLOR color) const
{
    switch(color)
    {
        case EGDC_3D_DARK_SHADOW:     return 0x77000000;
        case EGDC_3D_SHADOW:          return 0x77333333;
        case EGDC_3D_FACE:
        case EGDC_3D_HIGH_LIGHT:
        case EGDC_3D_LIGHT:
        case EGDC_ACTIVE_BORDER:
        case EGDC_ACTIVE_CAPTION:
        case EGDC_APP_WORKSPACE:
        case EGDC_BUTTON_TEXT:        return 0xffffffff;
        case EGDC_GRAY_TEXT:
        case EGDC_HIGH_LIGHT:
        case EGDC_HIGH_LIGHT_TEXT:
        case EGDC_INACTIVE_BORDER:
        case EGDC_INACTIVE_CAPTION:
        case EGDC_TOOLTIP:            return 0xffffffff;
        case EGDC_TOOLTIP_BACKGROUND: return 0x77000000;
        case EGDC_SCROLLBAR:
        case EGDC_WINDOW:
        case EGDC_WINDOW_SYMBOL:
        case EGDC_ICON:
        case EGDC_ICON_HIGH_LIGHT:    return 0xffffffff;
        default: return 0;
    }
}

void CSkin::setColor(EGUI_DEFAULT_COLOR which, SColor newColor)
{
    //...
}

s32 CSkin::getSize(EGUI_DEFAULT_SIZE size) const
{
    switch(size)
    {
        case EGDS_SCROLLBAR_SIZE:              return 12;
        case EGDS_MENU_HEIGHT:                 return 42;
        case EGDS_WINDOW_BUTTON_WIDTH:         return 20;
        case EGDS_CHECK_BOX_WIDTH:             return 10;
        case EGDS_MESSAGE_BOX_WIDTH:           return 0;
        case EGDS_MESSAGE_BOX_HEIGHT:          return 0;
        case EGDS_BUTTON_WIDTH:                return 32;
        case EGDS_BUTTON_HEIGHT:               return 32;
        case EGDS_TEXT_DISTANCE_X:             return 4;
        case EGDS_TEXT_DISTANCE_Y:             return 4;
        case EGDS_TITLEBARTEXT_DISTANCE_X:     return 8;
        case EGDS_TITLEBARTEXT_DISTANCE_Y:     return 2;
        case EGDS_MESSAGE_BOX_GAP_SPACE:       return 10;
        case EGDS_MESSAGE_BOX_MIN_TEXT_WIDTH:  return 4;
        case EGDS_MESSAGE_BOX_MAX_TEST_WIDTH:  return 10;
        case EGDS_MESSAGE_BOX_MIN_TEXT_HEIGHT: return 4;
        case EGDS_MESSAGE_BOX_MAX_TEXT_HEIGHT: return 10;
        default: return 0;
    }
}

const wchar_t* CSkin::getDefaultText(EGUI_DEFAULT_TEXT text) const
{
    switch(text)
    {
        case EGDT_MSG_BOX_OK:      return L"OK";
        case EGDT_MSG_BOX_CANCEL:  return L"Cancel";
        case EGDT_MSG_BOX_YES:     return L"Yes";
        case EGDT_MSG_BOX_NO:      return L"No";
        case EGDT_WINDOW_CLOSE:    return L"Close this window";
        case EGDT_WINDOW_MAXIMIZE: return L"Maximize this window";
        case EGDT_WINDOW_MINIMIZE: return L"Minimize this window";
        case EGDT_WINDOW_RESTORE:  return L"Restore this window";
        default: return L"";
    }
}

void CSkin::setDefaultText(EGUI_DEFAULT_TEXT which, const wchar_t* newText)
{
    //...
}

void CSkin::setSize(EGUI_DEFAULT_SIZE which, s32 size)
{
    //...
}

IGUIFont* CSkin::getFont(EGUI_DEFAULT_FONT which) const
{
    switch(which)
    {
        case EGDF_DEFAULT:
        case EGDF_BUTTON:
        case EGDF_WINDOW:
        case EGDF_MENU:    return m_normal_font;
        case EGDF_TOOLTIP: return m_mini_font;
        default: return NULL;
    }
}

void CSkin::setFont(IGUIFont* font, EGUI_DEFAULT_FONT which)
{
    //...
}

IGUISpriteBank* CSkin::getSpriteBank() const
{
    return m_sprites;
}

void CSkin::setSpriteBank(IGUISpriteBank* bank)
{
    //...
}

u32 CSkin::getIcon(EGUI_DEFAULT_ICON icon) const
{
    switch(icon)
    {
        case EGDI_WINDOW_MAXIMIZE:
        case EGDI_WINDOW_RESTORE:
        case EGDI_WINDOW_CLOSE:         return 0;
        case EGDI_WINDOW_MINIMIZE:
        case EGDI_WINDOW_RESIZE:
        case EGDI_CURSOR_UP:
        case EGDI_CURSOR_DOWN:
        case EGDI_CURSOR_LEFT:
        case EGDI_CURSOR_RIGHT:
        case EGDI_MENU_MORE:
        case EGDI_CHECK_BOX_CHECKED:
        case EGDI_DROP_DOWN:
        case EGDI_SMALL_CURSOR_UP:
        case EGDI_SMALL_CURSOR_DOWN:
        case EGDI_RADIO_BUTTON_CHECKED:
        case EGDI_MORE_LEFT:
        case EGDI_MORE_RIGHT:
        case EGDI_MORE_UP:
        case EGDI_MORE_DOWN: return 1;
        case EGDI_EXPAND:
        case EGDI_COLLAPSE:
        case EGDI_FILE:
        case EGDI_DIRECTORY:
        default: return 0;
    }
}

void CSkin::setIcon(EGUI_DEFAULT_ICON icon, u32 index)
{
    //...
}

void CSkin::draw3DButtonPaneStandard(IGUIElement* element,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip)
{
    DrawRoundedBox(rect, 0x7f333333);
}

void CSkin::draw3DButtonPanePressed(IGUIElement* element,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip)
{
    DrawRoundedBox(rect, 0x7f555555);
}

void CSkin::draw3DSunkenPane(IGUIElement* element,
    SColor bgcolor, bool flat, bool fillBackGround,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip)
{
    DrawRoundedBox(rect, 0x7f000000);
}

core::rect<s32> CSkin::draw3DWindowBackground(IGUIElement* element,
    bool drawTitleBar, SColor titleBarColor,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip,
    core::rect<s32>* checkClientArea)
{
    DrawRoundedBox(rect, 0x7f111111);

    core::rect<s32> titlebar(rect.UpperLeftCorner.X + 4, rect.UpperLeftCorner.Y + 2,
                             rect.LowerRightCorner.X - 4, rect.UpperLeftCorner.Y + 24);

    if(drawTitleBar)
        DrawRoundedBox(titlebar,
                       0x7f222222);

    return titlebar;
}

void CSkin::draw3DMenuPane(IGUIElement* element,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip)
{
    //...
}

void CSkin::draw3DToolBar(IGUIElement* element,
    const core::rect<s32>& rect,
    const core::rect<s32>* clip)
{
    m_device->getVideoDriver()->draw2DRectangle(0x7f000000, rect);
}

void CSkin::draw3DTabButton(IGUIElement* element, bool active,
    const core::rect<s32>& rect, const core::rect<s32>* clip, EGUI_ALIGNMENT alignment)
{
    //...
}

void CSkin::draw3DTabBody(IGUIElement* element, bool border, bool background,
    const core::rect<s32>& rect, const core::rect<s32>* clip, s32 tabHeight, EGUI_ALIGNMENT alignment )
{
    //...
}

void CSkin::drawIcon(IGUIElement* element, EGUI_DEFAULT_ICON icon,
    const position2di position, u32 starttime, u32 currenttime,
    bool loop, const core::rect<s32>* clip)
{
    //...
}

void CSkin::draw2DRectangle(IGUIElement* element, const SColor &color,
    const core::rect<s32>& pos, const core::rect<s32>* clip )
{
    //...
}

void CSkin::DrawRoundedBox(const core::rect<s32>& rect, const SColor& color) const
{
    // Make things easier by defining a few simple variables
    s32 x1 = rect.UpperLeftCorner.X;
    s32 y1 = rect.UpperLeftCorner.Y;
    s32 x2 = rect.LowerRightCorner.X;
    s32 y2 = rect.LowerRightCorner.Y;

    // Draw the four corners of the image
    m_device->getVideoDriver()->draw2DImage(m_tex,
                                            position2d<s32>(x1, y1),
                                            core::rect<s32>(0,0,8,8), 0, color, true);
    m_device->getVideoDriver()->draw2DImage(m_tex,
                                            position2d<s32>(x2 - 8, y1),
                                            core::rect<s32>(8,0,16,8), 0, color, true);
    m_device->getVideoDriver()->draw2DImage(m_tex,
                                            position2d<s32>(x1, y2 - 8),
                                            core::rect<s32>(0,8,8,16), 0, color, true);
    m_device->getVideoDriver()->draw2DImage(m_tex,
                                            position2d<s32>(x2 - 8, y2 - 8),
                                            core::rect<s32>(8,8,16,16), 0, color, true);
    // Now for the center
    m_device->getVideoDriver()->draw2DRectangle(color, core::rect<s32>(x1 + 8, y1 + 8, x2 - 8, y2 - 8));

    // Now the edges
    m_device->getVideoDriver()->draw2DRectangle(color, core::rect<s32>(x1 + 8, y1, x2 - 8, y1 + 8));
    m_device->getVideoDriver()->draw2DRectangle(color, core::rect<s32>(x1 + 8, y2 - 8, x2 - 8, y2));
    m_device->getVideoDriver()->draw2DRectangle(color, core::rect<s32>(x1, y1 + 8, x1 + 8, y2 - 8));
    m_device->getVideoDriver()->draw2DRectangle(color, core::rect<s32>(x2 - 8, y1 + 8, x2, y2 - 8));
}

