#include "CTrack.h"

// This is a global mapping of rotation values for each piece
CTrackOrientationInfo g_trackOrientationInfo[TT_NUM_PIECES] =
    {
        {     2, 1,   0,   1,   0,   0,   0,  0,  0 },   // TT_VERTICAL_STRAIGHT
        {     2, 1,   0,   1,   0,  90,   0,  0,  0 },   // TT_HORIZONTAL_STRAIGHT
        { 4.712, 0,   0,   2,   0,   0,   0, -2,  1 },   // TT_CURVE_NE
        { 4.712, 0,   0,   0,   0,  90,   0, -2, -1 },   // TT_CURVE_SE
        { 4.712, 2,   0,   0,   0, 180,   0,  2, -1 },   // TT_CURVE_SW
        { 4.712, 2,   0,   2,   0, 270,   0,  2,  1 },   // TT_CURVE_NW

        { 4.712, 0,   0,   0,   0,   0,   0,  1, -2 },   // TT_CURVE_NE_OTHER
        { 4.712, 0,   0,   0,   0,   0,   0,  1,  2 },   // TT_CURVE_SE_OTHER
        { 4.712, 0,   0,   0,   0,   0,   0, -1,  2 },   // TT_CURVE_SW_OTHER
        { 4.712, 0,   0,   0,   0,   0,   0, -1, -2 }    // TT_CURVE_NW_OTHER
    };

CTrack::CTrack(IrrlichtDevice * pDevice,float x, float y, float z, int ix, int iz)
    : m_type(TT_NUM_PIECES), m_x(x), m_y(y), m_z(z), m_offset_x(ix), m_offset_z(iz)
{
    // Create the node
    m_node = pDevice->getSceneManager()->addMeshSceneNode(NULL,0,-1,
                                                          core::vector3df(0, 0, 0),
                                                          core::vector3df(0, 0, 0),
                                                          core::vector3df(1, 1, 1),
                                                          true);

    // Now set the position
    m_node->setPosition(vector3df(m_x,m_y,m_z));
}

CTrack::~CTrack()
{
}

void CTrack::SetPiece(TRACK_TYPE type,IMesh * pMesh)
{
    if(pMesh)
        m_node->setVisible(true);
    else
        m_node->setVisible(false);

    m_type = type;

    // We need to update the mesh
    m_node->setMesh(pMesh);

    if(m_type != TT_NUM_PIECES)
    {
        // The rotation info needs updating too
        m_node->setRotation(vector3df(g_trackOrientationInfo[type].fRot_x,
                                      g_trackOrientationInfo[type].fRot_y,
                                      g_trackOrientationInfo[type].fRot_z));

        m_node->setPosition(vector3df(g_trackOrientationInfo[type].fTrans_x + m_x,
                                      g_trackOrientationInfo[type].fTrans_y + m_y,
                                      g_trackOrientationInfo[type].fTrans_z + m_z));

        m_node->setMaterialFlag(EMF_LIGHTING, false);
    }
}
