#include "CTrain.h"

CTrain::CTrain(IrrlichtDevice * pDevice, u32 t_time, CTrack * pTrack, std::string mesh_filename)
    : m_mesh(NULL), m_node(NULL), m_speed(0.0f), m_direction(D_NORTH), m_start_time(t_time), m_current_track(pTrack)
{
    // Try to open the train's mesh
    m_mesh = pDevice->getSceneManager()->getMesh(DATA_DIR "/models/simple_engine.b3d");

    // Grab a copy of it
    m_mesh->grab();

    // Now add the mesh to the scene manager
    m_node = pDevice->getSceneManager()->addMeshSceneNode(m_mesh);

    // Disable lighting on it for now
    //if(m_node)
    //    m_node->setMaterialFlag(EMF_LIGHTING, false);

    // TODO: add lookup for textures
}

CTrain::~CTrain()
{
    // Free the mesh
    m_mesh->drop();

    // Now remove this train
}

void CTrain::Process(CTrackManager * pManager,u32 t_time)
{
    // Calculate the percentage of the current piece of track
    // that the train has travelled.

    // Elapsed time
    int iElapsedMS = t_time - m_start_time;

    float fPerc = ((float)iElapsedMS / 1000.0f) * m_speed;

    // But wait! We might be at the end of the piece of track.
    // If so, get the next piece of track and set the percentage accordingly.
    TRACK_TYPE cur_type = m_current_track->GetPiece();

    if(fPerc > g_trackOrientationInfo[cur_type].length)
    {
        fPerc -= g_trackOrientationInfo[cur_type].length;

        m_start_time = t_time - (fPerc * 1000.0f);

        DIRECTION new_dir;
        m_current_track = pManager->GetNextTrack(m_current_track,m_direction,new_dir);
        m_direction = new_dir;
    }

    fPerc /= g_trackOrientationInfo[cur_type].length;

    float train_trans_x, train_trans_z;
    float train_rotate_y;

    // Get the offset for the piece of track
    pManager->CalculateTrainPosition(m_current_track,m_direction,fPerc,train_trans_x,train_trans_z,train_rotate_y);

    // Add the track's offset to this number
    train_trans_x += m_current_track->GetXPos();
    train_trans_z += m_current_track->GetZPos();

    // Now set this as the position
    m_node->setPosition(vector3df(train_trans_x,0,train_trans_z));
    m_node->setRotation(vector3df(0,train_rotate_y,0));
}
