// Standard includes
#include <iostream>

// Irrlicht includes
#include <irrlicht/irrlicht.h>

using namespace irr;

using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// OpenTracks includes
#include "CStateManager.h"
#include "CSkin.h"

int main(int argc, char** argv)
{
    // The first thing we need to do is attempt
    // to create the device. If we fail to do that,
    // then just report an immediate error and abort.

    // Determine what device we're to use
    E_DRIVER_TYPE driver_type = EDT_OPENGL;

    // TODO: Also determine what resolution and whether we're
    // to go fullscreen or not.
    IrrlichtDevice * pDevice =
        createDevice(driver_type, dimension2d<unsigned int>(960, 600), 16,
            false, false, true, NULL);

    // Check for failure
    if(!pDevice)
    {
        std::cerr << "There was an error initializing the Irrlicht "
                  << "video driver." << std::endl;
        return 1;
    }

    // Set the window title
    pDevice->setWindowCaption(L"OpenTracks - The Open Source Railway Simulator");

    // And activate our GUI skin
    pDevice->getGUIEnvironment()->setSkin(new CSkin(pDevice));

    // Now initialize the state manager, which we will then
    // hand over control of the application to.
    CStateManager state_manager(pDevice);

    // Release our handle to the device
    pDevice->drop();

    // Now hand off control to the state manager.
    state_manager.Go();

    // We're done now! Return 0
    return 0;
}

